@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+import pprint
 import urllib
 import urllib2
 
@@ -131,6 +132,8 @@ class Graph(object):
         return '<Graph(%r) at 0x%x>' % (str(self.url), id(self))
     
     def copy(self, **update):
+        """Copy this Graph, optionally overriding some attributes."""
+        
         return type(self)(access_token=self.access_token, **update)
     
     def __getitem__(self, item):
@@ -147,48 +150,147 @@ class Graph(object):
         
         if self.access_token:
             params['access_token'] = self.access_token
-        conn = urllib2.urlopen(self.url | params)
-        try:
-            data = json.load(conn)
-        finally:
-            conn.close()
         
-        if isinstance(data, dict):
-            return Node(self, bunch.bunchify(data))
-        return data
+        data = json.loads(self.fetch(self.url | params))
+        return Node._new(self, data)
     
     def fields(self, *fields):
+        """Shortcut for `?fields=x,y,z`."""
+        
         return self | ('fields', ','.join(fields))
     
     def ids(self, *ids):
+        """Shortcut for `?ids=1,2,3`."""
+        
         return self | ('ids', ','.join(map(str, ids)))
     
     def post(self, **params):
         
+        """
+        POST to this URL (with parameters); return the JSON-decoded result.
+        
+        Example:
+        
+            >>> Graph('ACCESS TOKEN').me.feed.post(message="Test.")
+            Node({'id': '...'})
+        
+        """
+        
         if self.access_token:
             params['access_token'] = self.access_token
-        conn = urllib2.urlopen(self.url, data=urllib.urlencode(params))
-        try:
-            data = json.load(conn)
-        finally:
-            conn.close()
         
-        if isinstance(data, dict):
-            return Node(self, bunch.bunchify(data))
-        return data
+        data = json.loads(self.fetch(self.url, data=urllib.urlencode(params)))
+        return Node._new(self, data)
     
     def delete(self):
+        """Delete this resource. Sends a POST with `?method=delete`."""
+        
         return self.post(method='delete')
+    
+    @staticmethod
+    def fetch(url, data=None):
+        
+        """
+        Fetch the specified URL, with optional form data; return a string.
+        
+        This method exists mainly for dependency injection purposes. By default
+        it uses urllib2; you may override it and use an alternative library.
+        """
+        
+        conn = urllib2.urlopen(url, data=data)
+        try:
+            return conn.read()
+        finally:
+            conn.close()
 
 
 class Node(bunch.Bunch):
     
+    """
+    Represent a JSON dictionary result from the Facebook Graph API.
+    
+    Accessing Items
+    ---------------
+    
+    You can access items using either attribute or item syntax:
+    
+        >>> n = Node._new(None, {'a': 1, 'b': {'c': 3}})
+        >>> n.a
+        1
+        >>> n['a']
+        1
+        >>> n.b.c
+        3
+        >>> n['b'].c
+        3
+        >>> n.b['c']
+        3
+        >>> n['b']['c']
+    
+    The same applies to assignment, although this is not recommended. Under the
+    hood, this uses `bunch.Bunch` (<http://pypi.python.org/pypi/bunch>).
+    
+    Requesting Child Nodes
+    ----------------------
+    
+    Many types of node in the Graph API have child nodes accessible through
+    further API calls. For example:
+    
+        >>> g = Graph('access token')
+        >>> g.me
+        <Graph('https://graph.facebook.com/me') at 0x...>
+        >>> g.me()  # User
+        Node(...)
+        >>> g.me.feed
+        <Graph('https://graph.facebook.com/me/feed) at 0x...>
+        >>> g.me.feed()  # News Feed for current user
+        Node(...)
+    
+    `Node` makes it easier to access these child nodes using dynamic attribute
+    handling:
+    
+        >>> me = g.me()
+        >>> me.feed
+        <Graph('https://graph.facebook.com/me/feed') at 0x...>
+    
+    Accessing a non-existent attribute or item will return a new `Graph`
+    pointing to the child node, which you can then retrieve (as a `Node`) by
+    calling directly.
+    
+    Because `Node` uses dynamic attribute handling, accessing non-existent
+    attributes will still return a `Graph`. This can sometimes seem like more of
+    a bug than a feature. You’ll have to check for the presence of a key first,
+    to make sure what you're trying to access actually exists:
+    
+        def user_location(user_id):
+            user = graph[user_id]()
+            if 'location' in user:
+                return user.location
+            return False
+    
+    """
+    
+    @classmethod
+    def _new(cls, api, data):
+        
+        """
+        Create a new `Node` from a `Graph` and a JSON-decoded object.
+        
+        If the object is not a dictionary, it will be returned as-is.
+        """
+        
+        if isinstance(data, dict):
+            return cls(api, bunch.bunchify(data))
+        return data
+    
     def __init__(self, api, data):
         super(Node, self).__init__(data)
         object.__setattr__(self, '_api', api)
     
     def __repr__(self):
-        return 'Node(%r)' % (dict(self),)
+        return 'Node(%r,\n%s)' % (
+            self._api,
+            indent(pprint.pformat(bunch.unbunchify(self)), prefix='     '))
     
     def __getitem__(self, item):
         try:
@@ -204,8 +306,18 @@ class Node(bunch.Bunch):
     
     @property
     def next_page(self):
+        """Shortcut for a `Graph` pointing to the next page."""
+        
         return self._api.copy(url=URLObject.parse(self.paging.next))
     
     @property
     def previous_page(self):
+        """Shortcut for a `Graph` pointing to the previous page."""
+        
         return self._api.copy(url=URLObject.parse(self.paging.next))
+
+
+def indent(string, prefix='    '):
+    """Indent each line of a string with a prefix (default is 4 spaces)."""
+    
+    return ''.join(prefix + line for line in string.splitlines(True))
